import os
import shutil
import requests
from PIL import Image
from resizeimage import resizeimage

from .google_api import GoogleCustomSearch


class FetchResizeSave(object):

    def __init__(self, developer_key, custom_search_cx):
        self._google_custom_search = GoogleCustomSearch(developer_key, custom_search_cx)
        self._search_resut = []

    def search(self, search_params, path_to_dir=False, width=None, height=None):
        for url in self._google_custom_search.search(search_params):

            image = GSImage(self)
            image.url = url

            if path_to_dir:
                image.download(path_to_dir)
                if width and height:
                    image.resize(width, height)

            self._search_resut.append(image)

    def results(self):
        return self._search_resut

    def download(self, url, path_to_dir):
        if not os.path.exists(path_to_dir):
            os.makedirs(path_to_dir)

        req = requests.get(url, stream=True)
        path_to_image = os.path.join(path_to_dir, url.split('/')[-1].split('?')[0])
        with open(path_to_image, 'wb') as f:
            req.raw.decode_content = True
            shutil.copyfileobj(req.raw, f)

        return path_to_image

    def resize(self, path_to_image, width, height):
        try:
            fd_img = open(path_to_image, 'rb')
            img = Image.open(fd_img)
            img = resizeimage.resize_cover(img, [int(width), int(height)])
            img.save(path_to_image, img.format)
            fd_img.close()
        except resizeimage.ImageSizeError as e:
            pass


class GSImage(object):

    def __init__(self, fetch_resize_save):
        self._fetch_resize_save = fetch_resize_save

        self._url = None
        self._path = None

        self.resized = False

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, image_url):
        self._url = image_url

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, image_path):
        self._path = image_path

    def download(self, path_to_dir):
        self._path = self._fetch_resize_save.download(self._url, path_to_dir)

    def resize(self, width, height):
        self._fetch_resize_save.resize(self._path, width, height)
        self.resized = True



import os
import requests
from apiclient.discovery import build


class GoogleCustomSearch(object):

    def __init__(self, developer_key=None, custom_search_cx=None):

        self._developer_key = developer_key or os.environ.get('GCS_DEVELOPER_KEY')
        self._custom_search_cx = custom_search_cx or os.environ.get('GCS_CX')

        self._google_build = None

        self._search_params_keys = {
            'q': None,
            'searchType': 'image',
            'num': 5,
            'imgType': 'photo',
            'imgSize': 'large',
            'fileType': 'jpg',
            'safe': 'off',
            'imgDominantColor': None
        }

    @property
    def google_service(self):
        if not self._google_build:
            self._google_build = build("customsearch", "v1", developerKey=self._developer_key)
        return self._google_build

    def search_params(self, params):
        search_params = {}

        for key, value in self._search_params_keys.items():
            if key in params:
                search_params[key] = params[key]
            else:
                if value:
                    search_params[key] = value

        return search_params

    def search(self, params):
        search_params = self.search_params(params)

        try:
            res = self.google_service.cse()\
                .list(cx=self._custom_search_cx, **search_params).execute()
        except:
            raise GoogleBackendException()

        for image in res.get('items'):
            try:
                check = requests.get(image['link'], timeout=5)
                if check.status_code == 200:
                    yield image['link']
            except:
                pass


class GoogleBackendException(Exception):
    pass
